# -*- coding: utf-8 -*-

"""Top-level package for SQL in F(unctions)."""

__author__ = """Digitalmensch"""
__email__ = 'contact@digitalmensch.ch'
__version__ = '0.2.5'

from .sqlfunc import sqldb
from .sqlf import sql
from .sqlf import scalar_udf
